@@ -5,7 +5,10 @@ import types
 
 from django.core.cache import cache
 from django.core.exceptions import ValidationError
-from django.core.urlresolvers import reverse
+try:
+    from django.urls import reverse
+except ImportError:
+    from django.core.urlresolvers import reverse
 from django.db import transaction, connection
 from django.db.models import Q, Value
 from django.db.models.functions import Concat, Substr
@@ -15,39 +18,35 @@ from django.utils.translation import ugettext_lazy as _
 from modeltranslation import settings as mt_settings
 from modeltranslation.translator import translator, NotRegistered
 from modeltranslation.utils import build_localized_fieldname, get_language
-from wagtail.contrib.settings.models import BaseSetting
-from wagtail.contrib.settings.views import get_setting_edit_handler
+
 try:
     from wagtail.contrib.routable_page.models import RoutablePageMixin
     from wagtail.admin.edit_handlers import FieldPanel, \
-        MultiFieldPanel, FieldRowPanel, InlinePanel, StreamFieldPanel, RichTextFieldPanel
+        MultiFieldPanel, FieldRowPanel, InlinePanel, StreamFieldPanel, RichTextFieldPanel,\
+        extract_panel_definitions_from_model_class, ObjectList
     from wagtail.core.models import Page, Site
     from wagtail.core.fields import StreamField, StreamValue
     from wagtail.core.url_routing import RouteResult
+    from wagtail.core.utils import WAGTAIL_APPEND_SLASH
     from wagtail.images.edit_handlers import ImageChooserPanel
     from wagtail.search.index import SearchField
-    from wagtail.snippets.models import get_snippet_models
     from wagtail.snippets.views.snippets import SNIPPET_EDIT_HANDLERS
 except ImportError:
     from wagtail.contrib.wagtailroutablepage.models import RoutablePageMixin
     from wagtail.wagtailadmin.edit_handlers import FieldPanel, \
-        MultiFieldPanel, FieldRowPanel, InlinePanel, StreamFieldPanel, RichTextFieldPanel
+        MultiFieldPanel, FieldRowPanel, InlinePanel, StreamFieldPanel, RichTextFieldPanel,\
+        extract_panel_definitions_from_model_class, ObjectList
     from wagtail.wagtailcore.models import Page, Site
     from wagtail.wagtailcore.fields import StreamField, StreamValue
     from wagtail.wagtailcore.url_routing import RouteResult
+    from wagtail.wagtailcore.utils import WAGTAIL_APPEND_SLASH
     from wagtail.wagtailimages.edit_handlers import ImageChooserPanel
     from wagtail.wagtailsearch.index import SearchField
-    from wagtail.wagtailsnippets.models import get_snippet_models
     from wagtail.wagtailsnippets.views.snippets import SNIPPET_EDIT_HANDLERS
-try:
-    from wagtail.core.utils import WAGTAIL_APPEND_SLASH
-except ImportError:
-    try:
-        from wagtail.wagtailcore.utils import WAGTAIL_APPEND_SLASH
-    except ImportError:
-        WAGTAIL_APPEND_SLASH = True  # Wagtail<1.5
-from wagtail_modeltranslation.settings import CUSTOM_SIMPLE_PANELS, CUSTOM_COMPOSED_PANELS
+from wagtail_modeltranslation.settings import CUSTOM_SIMPLE_PANELS, CUSTOM_COMPOSED_PANELS, TRANSLATE_SLUGS
 from wagtail_modeltranslation.utils import compare_class_tree_depth
+from wagtail_modeltranslation.patch_wagtailadmin_forms import WagtailFixedAdminPageForm
+from wagtail import VERSION
 
 logger = logging.getLogger('wagtail.core')
 
@@ -118,30 +117,38 @@ class WagtailTranslator(object):
                 descriptor = getattr(model, field.name)
                 _patch_stream_field_meaningful_value(descriptor)
 
-        # OVERRIDE PAGE METHODS
-        model.set_url_path = _new_set_url_path
-        model.route = _new_route
-        model._update_descendant_url_paths = _new_update_descendant_url_paths
-        if not hasattr(model, '_get_site_root_paths'):
-            model.get_url_parts = _new_get_url_parts  # Wagtail<1.11
-        model._get_site_root_paths = _new_get_site_root_paths
-        _patch_clean(model)
+        # OVERRIDE CLEAN METHOD
+        model.base_form_class = WagtailFixedAdminPageForm
 
-        if not model.save.__name__.startswith('localized'):
-            setattr(model, 'save', LocalizedSaveDescriptor(model.save))
+        # OVERRIDE PAGE METHODS
+        if TRANSLATE_SLUGS:
+            model.set_url_path = _new_set_url_path
+            model.route = _new_route
+            model._update_descendant_url_paths = _new_update_descendant_url_paths
+            if not hasattr(model, '_get_site_root_paths'):
+                model.get_url_parts = _new_get_url_parts  # Wagtail<1.11
+            model._get_site_root_paths = _new_get_site_root_paths
+            _patch_clean(model)
+
+            if not model.save.__name__.startswith('localized'):
+                setattr(model, 'save', LocalizedSaveDescriptor(model.save))
 
     def _patch_other_models(self, model):
         if hasattr(model, 'edit_handler'):
             edit_handler = model.edit_handler
-            for tab in edit_handler:
+            for tab in edit_handler.children:
                 tab.children = self._patch_panels(tab.children)
         elif hasattr(model, 'panels'):
             model.panels = self._patch_panels(model.panels)
-
-        if model in get_snippet_models() and model in SNIPPET_EDIT_HANDLERS:
-            del SNIPPET_EDIT_HANDLERS[model]
         else:
-            get_setting_edit_handler.cache_clear()
+            panels = extract_panel_definitions_from_model_class(model)
+            translation_registered_fields = translator.get_options_for_model(model).fields
+            panels = filter(lambda field: field.field_name not in translation_registered_fields, panels)
+            edit_handler = ObjectList(panels)
+            if VERSION < (2, 5):
+                SNIPPET_EDIT_HANDLERS[model] = edit_handler.bind_to_model(model)
+            else:
+                SNIPPET_EDIT_HANDLERS[model] = edit_handler.bind_to(model=model)
 
     def _patch_panels(self, panels_list, related_model=None):
         """
@@ -258,9 +265,9 @@ def _localized_set_url_path(page, parent, language):
         # for the current language. If the value for the current language is invalid we get the one
         # for the default fallback language
         slug = getattr(page, localized_slug_field, None) or \
-               getattr(page, default_localized_slug_field, None) or page.slug
+            getattr(page, default_localized_slug_field, None) or page.slug
         parent_url_path = getattr(parent, localized_url_path_field, None) or \
-                          getattr(parent, default_localized_url_path_field, None) or parent.url_path
+            getattr(parent, default_localized_url_path_field, None) or parent.url_path
 
         setattr(page, localized_url_path_field, parent_url_path + slug + '/')
 
@@ -455,7 +462,7 @@ def _update_translation_descendant_url_paths(old_record, page):
     default_localized_url_path = build_localized_fieldname('url_path', mt_settings.DEFAULT_LANGUAGE)
     for language in mt_settings.AVAILABLE_LANGUAGES:
         localized_url_path = build_localized_fieldname('url_path', language)
-        old_url_path = getattr(old_record, localized_url_path) or getattr(old_record, default_localized_url_path)
+        old_url_path = getattr(old_record, localized_url_path) or getattr(old_record, default_localized_url_path) or ''
         new_url_path = getattr(page, localized_url_path) or getattr(page, default_localized_url_path)
 
         if old_url_path == new_url_path:
@@ -569,5 +576,4 @@ def patch_wagtail_models():
     registered_models.sort(key=compare_class_tree_depth)
 
     for model_class in registered_models:
-        if issubclass(model_class, Page) or model_class in get_snippet_models() or issubclass(model_class, BaseSetting):
-            WagtailTranslator(model_class)
+        WagtailTranslator(model_class)
